package ru.ifmo.vizi.ctree.ui;

import ru.ifmo.vizi.base.ui.*;
import ru.ifmo.vizi.base.Configuration;

import java.awt.*;
import java.awt.event.ComponentEvent;
import java.util.Stack;
import java.util.Locale;

 /**
 * @author  Svyatoslav Pimenov
 * @version $Id: 0.0$
 */
public abstract class StringPanel extends Panel
{
    /**
     * Increase value button.
     */
    private final HintedButton btnMore;

    /**
     * Decrease value batton.
     */
    private final HintedButton btnLess;

    /**
     * Current value captionTemplate.
     */
    private final Label caption;

    /**
     * Panel hint.
     */
    private final String hint;

    /**
     * Array of strings.
     */
    private final Stack strings = new Stack();

    /**
     * Maximal allowed value.
     */
    private int nStrings;

    /**
     * Number of current string.
     */
    private int current;

    /**
     * Creates a new <code>StringPanel</code> with specified name.
     *
     * @param config configuration source.
     * @param name name of the parameter to get configuration from.
     */
    public StringPanel(Configuration config, String name, Locale locale)
    {
        setLayout(new BorderLayout());

        btnLess = new HintedButton(config, name + "-button-less")
        {
            protected void click()
            {
                current--;
                StringPanel.this.update(true);
            }
        };
        btnMore = new HintedButton(config, name + "-button-more")
        {
            protected void click()
            {
                current++;
                StringPanel.this.update(true);
            }
        };

        caption = new Label(null, Label.CENTER);

        add(btnLess, BorderLayout.WEST);
        add(caption, BorderLayout.CENTER);
        add(btnMore, BorderLayout.EAST);

        hint = config.getParameter(name + "-hint");
        Hinter.applyHint(caption, hint);

        enableEvents(AWTEvent.COMPONENT_EVENT_MASK);

        update(false);
    }

    /**
    * Add new string to this panel.
    *
    * @param s string to add.
    */
    public void addString(String s)
    {
        strings.push(s);
        nStrings++;
        update(false);
    }

    /**
     * Removes string at index index.
     *
     * @param index index of string to remove.
     */
    public void removeString(int index)
    {
        strings.removeElementAt(index);
        nStrings--;
        update(false);
    }

    /**
    * Gets current string.
    *
    * @return current string.
    */
    public String getCurrentString()
    {
        return new String((String) strings.elementAt(current));
    }

    /**
    * Gets number of current string.
    *
    * @return number of current string.
    */
    public int getCurrentIndex()
    {
        return current;
    }

    /**
     * Sets number of current string.
     *
     * @param index number of current string.
     */
    public void setCurrentIndex(int index)
    {
        current = index;
        update(false);
    }

    /**
    * Updates panel state.
    *
    * @param notify whether to notify user.
    */
    private void update(boolean notify)
    {
        if (current < 0)
        {
            current = 0;
        }
        if (current >= nStrings)
        {
            current = nStrings - 1;
        }

        btnLess.setEnabled(current > 0);
        btnMore.setEnabled(current < nStrings - 1);

        if (strings.empty())
        {
            caption.setText("Example");
        }
        else
        {
            caption.setText((String) strings.elementAt(current));
        }

        if (notify)
        {
            click(current);
        }
    }

    /**
     * Invoked when panel value changed.
     *
     * @param value new value.
     */
    protected abstract void click(double value);

    /**
     * Invoked when component state changes.
     *
     * @param e the component event
     */
    protected final void processComponentEvent(ComponentEvent e)
    {
        super.processComponentEvent(e);
        if (isVisible())
        {
            Hinter.applyHint(caption, hint);
        }
        else
        {
            Hinter.applyHint(caption, null);
        }
    }
}

